pub struct ShiftAmount(/* private fields */);Expand description
Represents an amount of bits to shift an ApInt.
The purpose of this type is to create a generic abstraction
over input types that may act as a ShiftAmount for shift
operations.
Implementations